package java.io;

import java.util.Iterator;
import java.util.Set;
import java.util.LinkedHashMap;
import java.util.Map;

class ExpiringCache
{
    private long millisUntilExpiration;
    private Map<String, ExpiringCache.Entry> map;
    private int queryCount;
    private int queryOverflow;
    private int MAX_ENTRIES;
    
    ExpiringCache() {
        this(30000L);
    }
    
    ExpiringCache(long millisUntilExpiration) {
        this.queryOverflow = 300;
        this.MAX_ENTRIES = 200;
        this.millisUntilExpiration = millisUntilExpiration;
        this.map = new LinkedHashMap<String, ExpiringCache.Entry>() {
            @Override
            protected boolean removeEldestEntry(Map.Entry<String, ExpiringCache.Entry> entry) {
                return this.size() > ExpiringCache.this.MAX_ENTRIES;
            }
        };
    }
    
    synchronized String get(String s) {
        if (++this.queryCount >= this.queryOverflow) {
            this.cleanup();
        }
        ExpiringCache.Entry entry = this.entryFor(s);
        if (entry != null) {
            return entry.val();
        }
        return null;
    }
    
    synchronized void put(String s, String val) {
        if (++this.queryCount >= this.queryOverflow) {
            this.cleanup();
        }
        ExpiringCache.Entry entry = this.entryFor(s);
        if (entry != null) {
            entry.setTimestamp(System.currentTimeMillis());
            entry.setVal(val);
        }
        else {
            this.map.put(s, new ExpiringCache.Entry(System.currentTimeMillis(), val));
        }
    }
    
    synchronized void clear() {
        this.map.clear();
    }
    
    private ExpiringCache.Entry entryFor(String s) {
        ExpiringCache.Entry entry = (ExpiringCache.Entry)this.map.get(s);
        if (entry != null) {
            long n = System.currentTimeMillis() - entry.timestamp();
            if (n < 0L || n >= this.millisUntilExpiration) {
                this.map.remove(s);
                entry = null;
            }
        }
        return entry;
    }
    
    private void cleanup() {
        Set<String> keySet = this.map.keySet();
        String[] array = new String[keySet.size()];
        int n = 0;
        Iterator<Object> iterator = keySet.iterator();
        while (iterator.hasNext()) {
            array[n++] = (String)iterator.next();
        }
        for (int i = 0; i < array.length; ++i) {
            this.entryFor(array[i]);
        }
        this.queryCount = 0;
    }
    
    static class Entry
    {
        private long timestamp;
        private String val;
        
        Entry(long timestamp, String val) {
            this.timestamp = timestamp;
            this.val = val;
        }
        
        long timestamp() {
            return this.timestamp;
        }
        
        void setTimestamp(long timestamp) {
            this.timestamp = timestamp;
        }
        
        String val() {
            return this.val;
        }
        
        void setVal(String val) {
            this.val = val;
        }
    }
}
